"use strict";
/*
 * Copyright (C) 2017, 2018 TypeFox and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const lsp = require("vscode-languageserver");
const lspTypeHierarchy = require("./type-hierarchy.lsp.proposal");
const protocol_translation_1 = require("./protocol-translation");
// tslint:disable-next-line:max-line-length
function computeTypeHierarchy(tspClient, documentProvider, params) {
    return __awaiter(this, void 0, void 0, function* () {
        const item = yield getItem(tspClient, params);
        if (item === null) {
            return null;
        }
        const direction = params.direction !== undefined ? params.direction : lspTypeHierarchy.TypeHierarchyDirection.Parents;
        const levelsToResolve = params.resolve || 0;
        yield resovleItem(tspClient, documentProvider, item, levelsToResolve, direction);
        return item;
    });
}
exports.computeTypeHierarchy = computeTypeHierarchy;
// tslint:disable-next-line:max-line-length
function resolveTypeHierarchy(tspClient, documentProvider, params) {
    return __awaiter(this, void 0, void 0, function* () {
        const item = params.item;
        yield resovleItem(tspClient, documentProvider, item, params.resolve, params.direction);
        return item;
    });
}
exports.resolveTypeHierarchy = resolveTypeHierarchy;
// tslint:disable-next-line:max-line-length
function resovleItem(tspClient, documentProvider, item, levelsToResolve, direction) {
    return __awaiter(this, void 0, void 0, function* () {
        if (levelsToResolve < 1) {
            return;
        }
        if (direction === lspTypeHierarchy.TypeHierarchyDirection.Parents || direction === lspTypeHierarchy.TypeHierarchyDirection.Both) {
            const parents = yield resolveParents(tspClient, documentProvider, item);
            item.parents = parents;
            for (const parent of parents) {
                yield resovleItem(tspClient, documentProvider, parent, levelsToResolve - 1, lspTypeHierarchy.TypeHierarchyDirection.Parents);
            }
        }
        if (direction === lspTypeHierarchy.TypeHierarchyDirection.Children || direction === lspTypeHierarchy.TypeHierarchyDirection.Both) {
            const children = yield resolveChildren(tspClient, documentProvider, item);
            item.children = children;
            for (const child of children) {
                yield resovleItem(tspClient, documentProvider, child, levelsToResolve - 1, lspTypeHierarchy.TypeHierarchyDirection.Children);
            }
        }
    });
}
function resolveParents(tspClient, documentProvider, item) {
    return __awaiter(this, void 0, void 0, function* () {
        const parents = [];
        const { uri, range } = item;
        const file = protocol_translation_1.uriToPath(uri);
        const document = documentProvider(file);
        if (!document) {
            return parents;
        }
        const start = document.offsetAt(range.start);
        const candidates = getCandidatesForSupertypesAtPosition(document.getText(), start);
        const candidatePositions = candidates.map(s => document.positionAt(s.start));
        for (const position of candidatePositions) {
            let definitionSymbol;
            let uri;
            const definitionReferences = yield findDefinitionReferences(tspClient, Object.assign({ file }, protocol_translation_1.asTextSpan(lsp.Range.create(position, position))));
            for (const definitionReference of definitionReferences) {
                definitionSymbol = yield findEnclosingSymbol(tspClient, definitionReference);
                if (definitionSymbol && isInterfaceOrClass(definitionSymbol)) {
                    uri = protocol_translation_1.pathToUri(definitionReference.file, undefined);
                    break;
                }
            }
            if (!definitionSymbol || !uri) {
                continue;
            }
            const { name, detail, kind, range, selectionRange } = definitionSymbol;
            parents.push({ uri, name, detail, kind, range, selectionRange });
        }
        return parents;
    });
}
function resolveChildren(tspClient, documentProvider, item) {
    return __awaiter(this, void 0, void 0, function* () {
        const result = [];
        const { uri, selectionRange } = item;
        const file = protocol_translation_1.uriToPath(uri);
        const references = yield findNonDefinitionReferences(tspClient, Object.assign({ file }, protocol_translation_1.asTextSpan(selectionRange)));
        for (const reference of references) {
            const symbol = yield findEnclosingSymbol(tspClient, reference);
            if (!symbol || !isInterfaceOrClass(symbol)) {
                continue;
            }
            const document = documentProvider(reference.file);
            if (!document) {
                return [];
            }
            const start = document.offsetAt(symbol.selectionRange.start);
            const candidates = getCandidatesForSupertypesAtPosition(document.getText(), start);
            if (candidates.some(c => c.symbol === item.name)) {
                const { name, detail, kind, range, selectionRange } = symbol;
                const uri = protocol_translation_1.pathToUri(reference.file, undefined);
                result.push({ uri, name, detail, kind, range, selectionRange });
            }
        }
        return result;
    });
}
function getItem(tspClient, params) {
    return __awaiter(this, void 0, void 0, function* () {
        let uri;
        let contextSymbol;
        const contextDefinitions = yield getDefinitions(tspClient, params);
        for (const contextDefinition of contextDefinitions) {
            uri = protocol_translation_1.pathToUri(contextDefinition.file, undefined);
            contextSymbol = yield findEnclosingSymbol(tspClient, contextDefinition);
            if (contextSymbol && isInterfaceOrClass(contextSymbol)) {
                break;
            }
            else {
                uri = undefined;
                contextSymbol = undefined;
            }
        }
        if (!contextSymbol || !uri) {
            return null;
        }
        const { name, detail, kind, range, selectionRange } = contextSymbol;
        if (!isInterfaceOrClass(contextSymbol)) {
            return null;
        }
        return { uri, name, detail, kind, range, selectionRange };
    });
}
function isInterfaceOrClass(symbol) {
    return symbol.kind === lsp.SymbolKind.Class || symbol.kind === lsp.SymbolKind.Interface;
}
function getDefinitions(tspClient, args) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = protocol_translation_1.uriToPath(args.textDocument.uri);
        if (!file) {
            return [];
        }
        const definitionResult = yield tspClient.request("definition" /* Definition */, {
            file,
            line: args.position.line + 1,
            offset: args.position.character + 1
        });
        const spans = definitionResult.body;
        return spans || [];
    });
}
function findEnclosingSymbol(tspClient, args) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = args.file;
        const response = yield tspClient.request("navtree" /* NavTree */, { file });
        const tree = response.body;
        if (!tree || !tree.childItems) {
            return undefined;
        }
        const pos = lsp.Position.create(args.start.line - 1, args.start.offset - 1);
        const symbol = yield findEnclosingSymbolInTree(tree, lsp.Range.create(pos, pos));
        return symbol;
    });
}
function findEnclosingSymbolInTree(parent, range) {
    return __awaiter(this, void 0, void 0, function* () {
        const inSpan = (span) => !!protocol_translation_1.Range.intersection(protocol_translation_1.asRange(span), range);
        const inTree = (tree) => tree.spans.some(span => inSpan(span));
        let candidate = inTree(parent) ? parent : undefined;
        outer: while (candidate) {
            const children = candidate.childItems || [];
            for (const child of children) {
                if (inTree(child)) {
                    candidate = child;
                    continue outer;
                }
            }
            break;
        }
        if (!candidate) {
            return undefined;
        }
        const span = candidate.spans.find(span => inSpan(span));
        const spanRange = protocol_translation_1.asRange(span);
        let selectionRange = spanRange;
        if (candidate.nameSpan) {
            const nameRange = protocol_translation_1.asRange(candidate.nameSpan);
            if (protocol_translation_1.Range.intersection(spanRange, nameRange)) {
                selectionRange = nameRange;
            }
        }
        return {
            name: candidate.text,
            kind: protocol_translation_1.toSymbolKind(candidate.kind),
            range: spanRange,
            selectionRange: selectionRange
        };
    });
}
function findDefinitionReferences(tspClient, args) {
    return __awaiter(this, void 0, void 0, function* () {
        return (yield findReferences(tspClient, args)).filter(ref => ref.isDefinition);
    });
}
function findNonDefinitionReferences(tspClient, args) {
    return __awaiter(this, void 0, void 0, function* () {
        return (yield findReferences(tspClient, args)).filter(ref => !ref.isDefinition);
    });
}
function findReferences(tspClient, args) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = args.file;
        const result = yield tspClient.request("references" /* References */, {
            file,
            line: args.start.line,
            offset: args.start.offset
        });
        if (!result.body) {
            return [];
        }
        return result.body.refs;
    });
}
function getCandidatesForSupertypesAtPosition(document, start) {
    const end = document.indexOf('{', start) - 1;
    let substring = document.substring(start, end);
    let match = /\Wextends\W/.exec(substring) || /\Wimplements\W/.exec(substring);
    if (!match) {
        return [];
    }
    start = start + match.index;
    substring = document.substring(start, end);
    function getSymbols(substring) {
        const result = [];
        const idRegex = /[$_a-zA-Z0-9\u{00C0}-\u{E007F}]/u;
        let start;
        let symbol = '';
        const pushChar = (i, c) => {
            if (start === undefined) {
                start = i;
            }
            symbol += c;
        };
        const pushSymbol = () => {
            if (start !== undefined) {
                result.push({ start, symbol });
                start = undefined;
                symbol = '';
            }
        };
        let bracketLevel = 0;
        for (let i = 0; i <= substring.length; i++) {
            const c = substring.charAt(i);
            if (c === '<') {
                pushSymbol();
                bracketLevel++;
            }
            if (c === '>') {
                bracketLevel = Math.max(bracketLevel - 1, 0);
            }
            if (bracketLevel === 0) {
                if (c.match(idRegex)) {
                    pushChar(i, c);
                }
                else {
                    pushSymbol();
                }
            }
        }
        pushSymbol();
        return result.filter(s => s.symbol !== 'implements' && s.symbol !== 'extends');
    }
    const result = getSymbols(substring).map(s => ({ start: start + s.start, symbol: s.symbol }));
    return result;
}
//# sourceMappingURL=type-hierarchy.js.map