"use strict";
/*
 * Copyright (C) 2017, 2018 TypeFox and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai = require("chai");
const lsp = require("vscode-languageserver");
const lspTypeHierarchy = require("./type-hierarchy.lsp.proposal");
const test_utils_1 = require("./test-utils");
const assert = chai.assert;
let diagnostics;
let server;
before(() => __awaiter(this, void 0, void 0, function* () {
    server = yield test_utils_1.createServer({
        rootUri: null,
        publishDiagnostics: args => diagnostics.push(args)
    });
}));
beforeEach(() => {
    diagnostics = [];
    server.closeAll();
});
describe('typeHierarchy', () => {
    function asString(item) {
        if (!item) {
            return '<not found>';
        }
        const symbolToString = (item) => `${item.name} (location: ${item.uri.split('/').pop()}#${item.selectionRange.start.line})`;
        const out = [];
        out.push(symbolToString(item));
        if (item.parents) {
            out.push(`[supertypes]`);
            for (const parent of item.parents) {
                out.push('--|> ' + symbolToString(parent));
            }
        }
        if (item.children) {
            out.push(`[subtypes]`);
            for (const child of item.children) {
                out.push('<|-- ' + symbolToString(child));
            }
        }
        if (item.parents) {
        }
        return out.join('\n').trim();
    }
    const textDocument = {
        uri: test_utils_1.uri('types.ts'),
        languageId: 'typescript',
        version: 1,
        text: `// comment on line 0
export interface SuperInterface {}
export interface SomeInterface {}
export interface Comparable extends SuperInterface {}
export class Bar implements Comparable {}
export class Foo extends Bar implements SomeInterface {}
export class Zoo extends Foo implements SuperInterface { /*
    ...
*/}`
    };
    const docModule1 = {
        uri: test_utils_1.uri('module1.ts'),
        languageId: 'typescript',
        version: 1,
        text: `// comment on line 0
export class Class1 {}
// const named equally
export const Interface1 = Symbol('Interface1');
export interface Interface1 { }`
    };
    const docModule2 = {
        uri: test_utils_1.uri('module2.ts'),
        languageId: 'typescript',
        version: 1,
        text: `// comment on line 0
import { Class1, Interface1 } from './module1';
export class Class2 extends Class1 {}
export class Class3 implements Interface1 {}`
    };
    function openDocuments() {
        const docs = [textDocument, docModule1, docModule2];
        for (const textDocument of docs) {
            server.didOpenTextDocument({ textDocument });
        }
    }
    it('find target symbol', () => __awaiter(this, void 0, void 0, function* () {
        openDocuments();
        const item = yield server.typeHierarchy({
            textDocument,
            position: lsp.Position.create(6, 15),
            direction: lspTypeHierarchy.TypeHierarchyDirection.Parents,
            resolve: 0
        });
        assert.equal(asString(item), `
Zoo (location: types.ts#6)`.trim());
    })).timeout(10000);
    it('supertypes: first level', () => __awaiter(this, void 0, void 0, function* () {
        openDocuments();
        const item = yield server.typeHierarchy({
            textDocument,
            position: lsp.Position.create(6, 15),
            direction: lspTypeHierarchy.TypeHierarchyDirection.Parents,
            resolve: 0
        });
        assert.isTrue(item !== null, "precondition failed: first level");
        assert.isTrue(item.parents === undefined, "precondition failed: unresolved item");
        const resolvedItem = yield server.typeHierarchyResolve({
            item: item,
            direction: lspTypeHierarchy.TypeHierarchyDirection.Parents,
            resolve: 1
        });
        assert.equal(asString(resolvedItem), `
Zoo (location: types.ts#6)
[supertypes]
--|> Foo (location: types.ts#5)
--|> SuperInterface (location: types.ts#1)`.trim());
    })).timeout(10000);
    it('supertypes: second level', () => __awaiter(this, void 0, void 0, function* () {
        openDocuments();
        const item = yield server.typeHierarchy({
            textDocument,
            position: lsp.Position.create(6, 15),
            direction: lspTypeHierarchy.TypeHierarchyDirection.Parents,
            resolve: 1
        });
        assert.isTrue(item !== null, "precondition failed: first level");
        assert.isTrue(item.parents !== undefined, "precondition failed: resolved item");
        const toBeResolved = item.parents[0];
        const resolvedItem = yield server.typeHierarchyResolve({
            item: toBeResolved,
            direction: lspTypeHierarchy.TypeHierarchyDirection.Parents,
            resolve: 1
        });
        assert.equal(asString(resolvedItem), `
Foo (location: types.ts#5)
[supertypes]
--|> Bar (location: types.ts#4)
--|> SomeInterface (location: types.ts#2)`.trim());
    })).timeout(10000);
    it('subtype: first level', () => __awaiter(this, void 0, void 0, function* () {
        openDocuments();
        const item = yield server.typeHierarchy({
            textDocument,
            position: lsp.Position.create(1, 20),
            direction: lspTypeHierarchy.TypeHierarchyDirection.Children,
            resolve: 0
        });
        assert.isTrue(item !== null, "precondition failed: first level");
        assert.isTrue(item.parents === undefined, "precondition failed: unresolved item");
        const resolvedItem = yield server.typeHierarchyResolve({
            item: item,
            direction: lspTypeHierarchy.TypeHierarchyDirection.Children,
            resolve: 1
        });
        assert.equal(asString(resolvedItem), `
SuperInterface (location: types.ts#1)
[subtypes]
<|-- Comparable (location: types.ts#3)
<|-- Zoo (location: types.ts#6)`.trim());
    })).timeout(10000);
    it('subtype: second level', () => __awaiter(this, void 0, void 0, function* () {
        openDocuments();
        const item = yield server.typeHierarchy({
            textDocument,
            position: lsp.Position.create(1, 20),
            direction: lspTypeHierarchy.TypeHierarchyDirection.Children,
            resolve: 1
        });
        assert.isTrue(item !== null, "precondition failed: first level");
        assert.isTrue(item.children !== undefined, "precondition failed: resolved item");
        const toBeResolved = item.children[0];
        const resolvedItem = yield server.typeHierarchyResolve({
            item: toBeResolved,
            direction: lspTypeHierarchy.TypeHierarchyDirection.Children,
            resolve: 1
        });
        assert.equal(asString(resolvedItem), `
Comparable (location: types.ts#3)
[subtypes]
<|-- Bar (location: types.ts#4)`.trim());
    })).timeout(10000);
    it('supertypes and subtypes combined', () => __awaiter(this, void 0, void 0, function* () {
        openDocuments();
        const item = yield server.typeHierarchy({
            textDocument,
            position: lsp.Position.create(5, 16),
            direction: lspTypeHierarchy.TypeHierarchyDirection.Both,
            resolve: 1
        });
        assert.isTrue(item !== null, "precondition failed: first level");
        assert.equal(asString(item), `
Foo (location: types.ts#5)
[supertypes]
--|> Bar (location: types.ts#4)
--|> SomeInterface (location: types.ts#2)
[subtypes]
<|-- Zoo (location: types.ts#6)`.trim());
    })).timeout(10000);
    it('supertype in imported module', () => __awaiter(this, void 0, void 0, function* () {
        openDocuments();
        const item = yield server.typeHierarchy({
            textDocument: docModule2,
            position: lsp.Position.create(2, 16),
            direction: lspTypeHierarchy.TypeHierarchyDirection.Parents,
            resolve: 1
        });
        assert.isTrue(item !== null, "precondition failed: first level");
        assert.equal(asString(item), `
Class2 (location: module2.ts#2)
[supertypes]
--|> Class1 (location: module1.ts#1)`.trim());
    })).timeout(10000);
    it('subtype in imported module', () => __awaiter(this, void 0, void 0, function* () {
        openDocuments();
        const item = yield server.typeHierarchy({
            textDocument: docModule1,
            position: lsp.Position.create(1, 16),
            direction: lspTypeHierarchy.TypeHierarchyDirection.Children,
            resolve: 1
        });
        assert.isTrue(item !== null, "precondition failed: first level");
        assert.equal(asString(item), `
Class1 (location: module1.ts#1)
[subtypes]
<|-- Class2 (location: module2.ts#2)`.trim());
    })).timeout(10000);
    it('subtypes of interfaces', () => __awaiter(this, void 0, void 0, function* () {
        openDocuments();
        const item = yield server.typeHierarchy({
            textDocument: docModule1,
            position: lsp.Position.create(4, 20),
            direction: lspTypeHierarchy.TypeHierarchyDirection.Children,
            resolve: 1
        });
        assert.isTrue(item !== null, "precondition failed: first level");
        assert.equal(asString(item), `
Interface1 (location: module1.ts#4)
[subtypes]
<|-- Class3 (location: module2.ts#3)`.trim());
    })).timeout(10000);
});
//# sourceMappingURL=type-hierarchy.spec.js.map